@@ -2,9 +2,10 @@
 
 from collections.abc import Sequence
 
-from mkosi.config import Architecture
+from mkosi.config import Architecture, Config
 from mkosi.context import Context
 from mkosi.distributions import DistributionInstaller
+from mkosi.installer import PackageManager
 from mkosi.log import die
 
 
@@ -13,6 +14,10 @@ class Installer(DistributionInstaller):
     def architecture(cls, arch: Architecture) -> str:
         return str(arch)
 
+    @classmethod
+    def package_manager(cls, config: Config) -> type[PackageManager]:
+        return PackageManager
+
     @classmethod
     def setup(cls, context: Context) -> None:
         pass
