@@ -1,4 +1,5 @@
 # SPDX-License-Identifier: LGPL-2.1+
+from collections.abc import Iterator
 from typing import Any, cast
 
 import pytest
@@ -6,7 +7,7 @@ import pytest
 from mkosi.config import parse_config
 from mkosi.distributions import Distribution, detect_distribution
 
-from . import Image
+from . import Image, ci_group
 
 
 def pytest_addoption(parser: Any) -> None:
@@ -33,6 +34,16 @@ def pytest_addoption(parser: Any) -> None:
         type=Distribution,
         choices=[Distribution(d) for d in Distribution.values()],
     )
+    parser.addoption(
+        "--tools-tree-release",
+        metavar="RELEASE",
+        help="Use the given tools tree release instead of the default one",
+    )
+    parser.addoption(
+        "--debug-shell",
+        help="Pass --debug-shell when running mkosi",
+        action="store_true",
+    )
 
 
 @pytest.fixture(scope="session")
@@ -43,4 +54,12 @@ def config(request: Any) -> Image.Config:
         distribution=distribution,
         release=release,
         tools_tree_distribution=cast(Distribution, request.config.getoption("--tools-tree-distribution")),
+        tools_tree_release=request.config.getoption("--tools-tree-release"),
+        debug_shell=request.config.getoption("--debug-shell"),
     )
+
+
+@pytest.fixture(autouse=True)
+def ci_sections(request: Any) -> Iterator[None]:
+    with ci_group(request.node.name):
+        yield
