@@ -10,6 +10,7 @@ from typing import Optional
 
 import pytest
 
+from mkosi import expand_kernel_specifiers
 from mkosi.config import (
     Architecture,
     Compression,
@@ -84,8 +85,8 @@ def test_parse_ini(tmp_path: Path) -> None:
     assert next(g) == ("MySection", "Value", "abc")
     assert next(g) == ("MySection", "Other", "def")
     assert next(g) == ("MySection", "ALLCAPS", "txt")
-    assert next(g) == ("", "", "")
-    assert next(g) == ("", "", "")
+    assert next(g) == ("MySection", "", "")
+    assert next(g) == ("EmptySection", "", "")
     assert next(g) == ("AnotherSection", "EmptyValue", "")
     assert next(g) == ("AnotherSection", "Multiline", "abc\ndef\nqed\nord")
 
@@ -221,6 +222,43 @@ def test_parse_config(tmp_path: Path) -> None:
         assert config.split_artifacts is False
 
 
+def test_parse_includes_once(tmp_path: Path) -> None:
+    d = tmp_path
+
+    (d / "mkosi.conf").write_text(
+        """\
+        [Content]
+        Bootable=yes
+        BuildPackages=abc
+        """
+    )
+    (d / "abc.conf").write_text(
+        """\
+        [Content]
+        BuildPackages=def
+        """
+    )
+
+    with chdir(d):
+        _, [config] = parse_config(["--include", "abc.conf", "--include", "abc.conf"])
+        assert config.build_packages == ["def", "abc"]
+
+    (d / "mkosi.images").mkdir()
+
+    for n in ("one", "two"):
+        (d / "mkosi.images" / f"{n}.conf").write_text(
+            """\
+            [Config]
+            Include=abc.conf
+            """
+        )
+
+    with chdir(d):
+        _, [one, two] = parse_config([])
+        assert one.build_packages == ["abc", "def"]
+        assert two.build_packages == ["abc", "def"]
+
+
 def test_profiles(tmp_path: Path) -> None:
     d = tmp_path
 
@@ -307,8 +345,6 @@ def test_parse_load_verb(tmp_path: Path) -> None:
     with chdir(tmp_path):
         assert parse_config(["build"])[0].verb == Verb.build
         assert parse_config(["clean"])[0].verb == Verb.clean
-        with pytest.raises(SystemExit):
-            parse_config(["help"])
         assert parse_config(["genkey"])[0].verb == Verb.genkey
         assert parse_config(["bump"])[0].verb == Verb.bump
         assert parse_config(["serve"])[0].verb == Verb.serve
@@ -357,6 +393,28 @@ def test_compression(tmp_path: Path) -> None:
         assert config.compress_output == Compression.none
 
 
+def test_match_only(tmp_path: Path) -> None:
+    with chdir(tmp_path):
+        Path("mkosi.conf").write_text(
+            """\
+            [Match]
+            Format=|directory
+            Format=|disk
+            """
+        )
+
+        Path("mkosi.conf.d").mkdir()
+        Path("mkosi.conf.d/10-abc.conf").write_text(
+            """\
+            [Output]
+            ImageId=abcde
+            """
+        )
+
+        _, [config] = parse_config(["--format", "tar"])
+        assert config.image_id != "abcde"
+
+
 def test_match_multiple(tmp_path: Path) -> None:
     with chdir(tmp_path):
         Path("mkosi.conf").write_text(
@@ -386,6 +444,105 @@ def test_match_multiple(tmp_path: Path) -> None:
         _, [config] = parse_config(["--format", "disk", "--architecture", "x86-64"])
         assert config.image_id == "abcde"
 
+        Path("mkosi.conf").write_text(
+            """\
+            [TriggerMatch]
+            Format=disk
+            Architecture=x86-64
+
+            [TriggerMatch]
+            Format=directory
+            Architecture=arm64
+
+            [Output]
+            ImageId=abcde
+            """
+        )
+
+        # Both sections are not matched, so image ID should not be "abcde".
+        _, [config] = parse_config(["--format", "tar", "--architecture", "s390x"])
+        assert config.image_id != "abcde"
+
+        # The first section is matched, so image ID should be "abcde".
+        _, [config] = parse_config(["--format", "disk", "--architecture", "x86-64"])
+        assert config.image_id == "abcde"
+
+        # The second section is matched, so image ID should be "abcde".
+        _, [config] = parse_config(["--format", "directory", "--architecture", "arm64"])
+        assert config.image_id == "abcde"
+
+        # Parts of all section are matched, but none is matched fully, so image ID should not be "abcde".
+        _, [config] = parse_config(["--format", "disk", "--architecture", "arm64"])
+        assert config.image_id != "abcde"
+
+        Path("mkosi.conf").write_text(
+            """\
+            [TriggerMatch]
+            Format=|disk
+            Format=|directory
+
+            [TriggerMatch]
+            Format=directory
+            Architecture=arm64
+
+            [Output]
+            ImageId=abcde
+            """
+        )
+
+        # The first section is matched, so image ID should be "abcde".
+        _, [config] = parse_config(["--format", "disk"])
+        assert config.image_id == "abcde"
+
+        Path("mkosi.conf").write_text(
+            """\
+            [TriggerMatch]
+            Format=|disk
+            Format=|directory
+            Architecture=x86-64
+
+            [TriggerMatch]
+            Format=directory
+            Architecture=arm64
+
+            [Output]
+            ImageId=abcde
+            """
+        )
+
+        # No sections are matched, so image ID should be not "abcde".
+        _, [config] = parse_config(["--format", "disk", "--architecture=arm64"])
+        assert config.image_id != "abcde"
+
+        # Mixing both [Match] and [TriggerMatch]
+        Path("mkosi.conf").write_text(
+            """\
+            [Match]
+            Format=disk
+
+            [TriggerMatch]
+            Architecture=arm64
+
+            [TriggerMatch]
+            Architecture=x86-64
+
+            [Output]
+            ImageId=abcde
+            """
+        )
+
+        # Match and first TriggerMatch sections match
+        _, [config] = parse_config(["--format", "disk", "--architecture=arm64"])
+        assert config.image_id == "abcde"
+
+        # Match section matches, but no TriggerMatch section matches
+        _, [config] = parse_config(["--format", "disk", "--architecture=s390x"])
+        assert config.image_id != "abcde"
+
+        # Second TriggerMatch section matches, but the Match section does not
+        _, [config] = parse_config(["--format", "tar", "--architecture=x86-64"])
+        assert config.image_id != "abcde"
+
 
 @pytest.mark.parametrize("dist1,dist2", itertools.combinations_with_replacement(Distribution, 2))
 def test_match_distribution(tmp_path: Path, dist1: Distribution, dist2: Distribution) -> None:
@@ -656,6 +813,48 @@ def test_match_imageversion(tmp_path: Path, op: str, version: str) -> None:
         assert "testpkg3" not in conf.packages
 
 
+def test_match_environment(tmp_path: Path) -> None:
+    d = tmp_path
+
+    (d / "mkosi.conf").write_text(
+        """\
+        [Match]
+        Environment=MYENV=abc
+
+        [Content]
+        ImageId=matched
+        """
+    )
+
+    with chdir(d):
+        _, [conf] = parse_config(["--environment", "MYENV=abc"])
+        assert conf.image_id == "matched"
+        _, [conf] = parse_config(["--environment", "MYENV=abd"])
+        assert conf.image_id != "matched"
+        _, [conf] = parse_config(["--environment", "MYEN=abc"])
+        assert conf.image_id != "matched"
+        _, [conf] = parse_config(["--environment", "MYEN=abd"])
+        assert conf.image_id != "matched"
+
+    (d / "mkosi.conf").write_text(
+        """\
+        [Match]
+        Environment=MYENV
+
+        [Content]
+        ImageId=matched
+        """
+    )
+
+    with chdir(d):
+        _, [conf] = parse_config(["--environment", "MYENV=abc"])
+        assert conf.image_id == "matched"
+        _, [conf] = parse_config(["--environment", "MYENV=abd"])
+        assert conf.image_id == "matched"
+        _, [conf] = parse_config(["--environment", "MYEN=abc"])
+        assert conf.image_id != "matched"
+
+
 @pytest.mark.parametrize(
     "skel,pkgmngr", itertools.product(
         [None, Path("/foo"), Path("/bar")],
@@ -764,6 +963,28 @@ def test_specifiers(tmp_path: Path) -> None:
                     ImageVersion=%v
                     OutputDirectory=%O
                     Output=%o
+                    ConfigRootDirectory=%D
+                    ConfigRootConfdir=%C
+                    ConfigRootPwd=%P
+        """
+    )
+
+    (d / "mkosi.conf.d").mkdir()
+    (d / "mkosi.conf.d/abc.conf").write_text(
+        """\
+        [Content]
+        Environment=ConfigAbcDirectory=%D
+                    ConfigAbcConfdir=%C
+                    ConfigAbcPwd=%P
+        """
+    )
+    (d / "mkosi.conf.d/qed").mkdir()
+    (d / "mkosi.conf.d/qed/mkosi.conf").write_text(
+        """
+        [Content]
+        Environment=ConfigQedDirectory=%D
+                    ConfigQedConfdir=%C
+                    ConfigQedPwd=%P
         """
     )
 
@@ -778,11 +999,45 @@ def test_specifiers(tmp_path: Path) -> None:
             "ImageVersion": "1.2.3",
             "OutputDirectory": str(Path.cwd() / "abcde"),
             "Output": "test",
+            "ConfigRootDirectory": os.fspath(d),
+            "ConfigRootConfdir": os.fspath(d),
+            "ConfigRootPwd": os.fspath(d),
+            "ConfigAbcDirectory": os.fspath(d),
+            "ConfigAbcConfdir": os.fspath(d / "mkosi.conf.d"),
+            "ConfigAbcPwd": os.fspath(d),
+            "ConfigQedDirectory": os.fspath(d),
+            "ConfigQedConfdir": os.fspath(d / "mkosi.conf.d/qed"),
+            "ConfigQedPwd": os.fspath(d / "mkosi.conf.d/qed"),
         }
 
         assert {k: v for k, v in config.environment.items() if k in expected} == expected
 
 
+def test_kernel_specifiers(tmp_path: Path) -> None:
+    kver = "13.0.8-5.10.0-1057-oem"     # taken from reporter of #1638
+    token = "MySystemImage"
+    roothash = "67e893261799236dcf20529115ba9fae4fd7c2269e1e658d42269503e5760d38"
+    boot_count = "3"
+
+    def test_expand_kernel_specifiers(text: str) -> str:
+        return expand_kernel_specifiers(
+            text,
+            kver=kver,
+            token=token,
+            roothash=roothash,
+            boot_count=boot_count,
+        )
+
+    assert test_expand_kernel_specifiers("&&") == "&"
+    assert test_expand_kernel_specifiers("&k") == kver
+    assert test_expand_kernel_specifiers("&e") == token
+    assert test_expand_kernel_specifiers("&h") == roothash
+    assert test_expand_kernel_specifiers("&c") == boot_count
+
+    assert test_expand_kernel_specifiers("Image_1.0.3") == "Image_1.0.3"
+    assert test_expand_kernel_specifiers("Image~&c+&h-&k-&e") == f"Image~{boot_count}+{roothash}-{kver}-{token}"
+
+
 def test_output_id_version(tmp_path: Path) -> None:
     d = tmp_path
 
